"""
Scapy RPC definitions
"""

import importlib
import importlib.machinery
import pathlib

__version__ = "0.0.1"


def scapy_ext(plg):
    plg.config("Scapy RPC", __version__)
    for lay in pathlib.Path(__file__).parent.glob('*.py'):
        if lay.name == "__init__.py":
            continue
        plg.register(
            name="msrpce.raw." + lay.name[:-3],
            mode=plg.MODE.LAYERS,
            path=lay.absolute(),
        )



### ms-mqmp.idl
"""
RPC definitions for the following interfaces:
- qmcomm (v1.0): fdb3a030-065f-11d1-bb9b-00a024ea5525
- qmcomm2 (v1.0): 76d12b80-3467-11d3-91ff-0090272f9ea3
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarFieldListField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRRecursiveField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    NDRVarStrLenField,
    NDRVarStrLenFieldUtf16,
    register_dcerpc_interface,
)


class R_QMGetRemoteQueueName_Request(NDRPacket):
    fields_desc = [
        NDRIntField("pQueue", 0),
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpRemoteQueueName", ""))
        ),
    ]


class R_QMGetRemoteQueueName_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpRemoteQueueName", ""))
        ),
        NDRIntField("status", 0),
    ]


class QUEUE_FORMAT_TYPE(IntEnum):
    QUEUE_FORMAT_TYPE_UNKNOWN = 0
    QUEUE_FORMAT_TYPE_PUBLIC = 1
    QUEUE_FORMAT_TYPE_PRIVATE = 2
    QUEUE_FORMAT_TYPE_DIRECT = 3
    QUEUE_FORMAT_TYPE_MACHINE = 4
    QUEUE_FORMAT_TYPE_CONNECTOR = 5
    QUEUE_FORMAT_TYPE_DL = 6
    QUEUE_FORMAT_TYPE_MULTICAST = 7
    QUEUE_FORMAT_TYPE_SUBQUEUE = 8


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class OBJECTID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Lineage", GUID(), GUID),
        NDRIntField("Uniquifier", 0),
    ]


class DL_ID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("m_DlGuid", GUID(), GUID),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("m_pwzDomain", ""), deferred=True
        ),
    ]


class MULTICAST_ID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("m_address", 0), NDRIntField("m_port", 0)]


class QUEUE_FORMAT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("m_qft", 0),
        NDRByteField("m_SuffixAndFlags", 0),
        NDRShortField("m_reserved", 0),
        NDRUnionField(
            [
                (
                    StrFixedLenField("value", "", length=0),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_UNKNOWN
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PUBLIC
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", OBJECTID(), OBJECTID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_PRIVATE
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DIRECT
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MACHINE
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_CONNECTOR
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", DL_ID(), DL_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_DL
                        ),
                    ),
                ),
                (
                    NDRPacketField("value", MULTICAST_ID(), MULTICAST_ID),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_MULTICAST
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRConfVarStrNullFieldUtf16("value", ""), deferred=True
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "m_qft", None)
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                        (
                            lambda _, val: val.tag
                            == QUEUE_FORMAT_TYPE.QUEUE_FORMAT_TYPE_SUBQUEUE
                        ),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(1, 8),
            switch_fmt=("B", "B"),
        ),
    ]


class R_QMOpenRemoteQueue_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT)
        ),
        NDRIntField("dwCallingProcessID", 0),
        NDRIntField("dwDesiredAccess", 0),
        NDRIntField("dwShareMode", 0),
        NDRPacketField("pLicGuid", GUID(), GUID),
        NDRIntField("dwMQS", 0),
    ]


class R_QMOpenRemoteQueue_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("pdwContext", 0),
        NDRIntField("dwpQueue", 0),
        NDRIntField("phQueue", 0),
        NDRIntField("status", 0),
    ]


class R_QMCloseRemoteQueueContext_Request(NDRPacket):
    fields_desc = [NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle)]


class R_QMCloseRemoteQueueContext_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class TRANSFER_TYPE(IntEnum):
    CACTB_SEND = 0
    CACTB_RECEIVE = 1
    CACTB_CREATECURSOR = 2


class Send_sub0(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pAdminQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField("pResponseQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
            deferred=True,
        ),
    ]


class Receive_sub1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("RequestTimeout", 0),
        NDRIntField("Action", 0),
        NDRIntField("Asynchronous", 0),
        NDRIntField("Cursor", 0),
        NDRIntField("ulResponseFormatNameLen", None, size_of="ppResponseFormatName"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppResponseFormatName",
                [],
                NDRFullPointerField(
                    NDRShortField("ppResponseFormatName", 0), deferred=True
                ),
                size_is=lambda pkt: pkt.ulResponseFormatNameLen,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRIntField("pulResponseFormatNameLenProp", 0), deferred=True
        ),
        NDRIntField("ulAdminFormatNameLen", None, size_of="ppAdminFormatName"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppAdminFormatName",
                [],
                NDRFullPointerField(
                    NDRShortField("ppAdminFormatName", 0), deferred=True
                ),
                size_is=lambda pkt: pkt.ulAdminFormatNameLen,
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRIntField("pulAdminFormatNameLenProp", 0), deferred=True),
        NDRIntField("ulDestFormatNameLen", None, size_of="ppDestFormatName"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppDestFormatName",
                [],
                NDRFullPointerField(
                    NDRShortField("ppDestFormatName", 0), deferred=True
                ),
                size_is=lambda pkt: pkt.ulDestFormatNameLen,
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRIntField("pulDestFormatNameLenProp", 0), deferred=True),
        NDRIntField("ulOrderingFormatNameLen", None, size_of="ppOrderingFormatName"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppOrderingFormatName",
                [],
                NDRFullPointerField(
                    NDRShortField("ppOrderingFormatName", 0), deferred=True
                ),
                size_is=lambda pkt: pkt.ulOrderingFormatNameLen,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRIntField("pulOrderingFormatNameLenProp", 0), deferred=True
        ),
    ]


class CACCreateRemoteCursor(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("hCursor", 0),
        NDRIntField("srv_hACQueue", 0),
        NDRIntField("cli_pQMQueue", 0),
    ]


class XACTUOW(NDRPacket):
    fields_desc = [StrFixedLenField("rgb", "", length=16)]


class CACTransferBufferV1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("uTransferType", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField("value", Send_sub0(), Send_sub0),
                    (
                        (
                            lambda pkt: getattr(pkt, "uTransferType", None)
                            == TRANSFER_TYPE.CACTB_SEND
                        ),
                        (lambda _, val: val.tag == TRANSFER_TYPE.CACTB_SEND),
                    ),
                ),
                (
                    NDRPacketField("value", Receive_sub1(), Receive_sub1),
                    (
                        (
                            lambda pkt: getattr(pkt, "uTransferType", None)
                            == TRANSFER_TYPE.CACTB_RECEIVE
                        ),
                        (lambda _, val: val.tag == TRANSFER_TYPE.CACTB_RECEIVE),
                    ),
                ),
                (
                    NDRPacketField(
                        "value", CACCreateRemoteCursor(), CACCreateRemoteCursor
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "uTransferType", None)
                            == TRANSFER_TYPE.CACTB_CREATECURSOR
                        ),
                        (lambda _, val: val.tag == TRANSFER_TYPE.CACTB_CREATECURSOR),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRFullPointerField(NDRShortField("pClass", 0), deferred=True),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppMessageID", OBJECTID(), OBJECTID), deferred=True
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfVarFieldListField(
                "ppCorrelationID",
                [],
                NDRFullPointerField(
                    NDRVarStrLenField("ppCorrelationID", "", length_is=lambda pkt: 20),
                    deferred=True,
                ),
                size_is=lambda pkt: 20,
                length_is=lambda pkt: 20,
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRIntField("pSentTime", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pArrivedTime", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pPriority", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pDelivery", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pAcknowledge", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pAuditing", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pApplicationTag", 0), deferred=True),
        NDRFullPointerField(
            NDRConfVarFieldListField(
                "ppBody",
                [],
                NDRFullPointerField(
                    NDRVarStrLenField(
                        "ppBody", "", length_is=lambda pkt: pkt.ulBodyBufferSizeInBytes
                    ),
                    deferred=True,
                ),
                size_is=lambda pkt: pkt.ulAllocBodyBufferInBytes,
                length_is=lambda pkt: pkt.ulBodyBufferSizeInBytes,
            ),
            deferred=True,
        ),
        NDRIntField("ulBodyBufferSizeInBytes", None, size_of="ppBody"),
        NDRIntField("ulAllocBodyBufferInBytes", None, size_of="ppBody"),
        NDRFullPointerField(NDRIntField("pBodySize", 0), deferred=True),
        NDRFullPointerField(
            NDRConfVarFieldListField(
                "ppTitle",
                [],
                NDRFullPointerField(
                    NDRVarStrLenFieldUtf16(
                        "ppTitle",
                        "",
                        length_is=lambda pkt: pkt.ulTitleBufferSizeInWCHARs,
                    ),
                    deferred=True,
                ),
                size_is=lambda pkt: pkt.ulTitleBufferSizeInWCHARs,
                length_is=lambda pkt: pkt.ulTitleBufferSizeInWCHARs,
            ),
            deferred=True,
        ),
        NDRIntField("ulTitleBufferSizeInWCHARs", None, size_of="ppTitle"),
        NDRFullPointerField(
            NDRIntField("pulTitleBufferSizeInWCHARs", 0), deferred=True
        ),
        NDRIntField("ulAbsoluteTimeToQueue", 0),
        NDRFullPointerField(NDRIntField("pulRelativeTimeToQueue", 0), deferred=True),
        NDRIntField("ulRelativeTimeToLive", 0),
        NDRFullPointerField(NDRIntField("pulRelativeTimeToLive", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pTrace", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulSenderIDType", 0), deferred=True),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppSenderID",
                [],
                NDRFullPointerField(NDRByteField("ppSenderID", 0), deferred=True),
                size_is=lambda pkt: pkt.uSenderIDLen,
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRIntField("pulSenderIDLenProp", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulPrivLevel", 0), deferred=True),
        NDRIntField("ulAuthLevel", 0),
        NDRFullPointerField(NDRByteField("pAuthenticated", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulHashAlg", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulEncryptAlg", 0), deferred=True),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppSenderCert",
                [],
                NDRFullPointerField(NDRByteField("ppSenderCert", 0), deferred=True),
                size_is=lambda pkt: pkt.ulSenderCertLen,
            ),
            deferred=True,
        ),
        NDRIntField("ulSenderCertLen", None, size_of="ppSenderCert"),
        NDRFullPointerField(NDRIntField("pulSenderCertLenProp", 0), deferred=True),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppwcsProvName",
                [],
                NDRFullPointerField(NDRShortField("ppwcsProvName", 0), deferred=True),
                size_is=lambda pkt: pkt.ulProvNameLen,
            ),
            deferred=True,
        ),
        NDRIntField("ulProvNameLen", None, size_of="ppwcsProvName"),
        NDRFullPointerField(NDRIntField("pulAuthProvNameLenProp", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulProvType", 0), deferred=True),
        NDRSignedIntField("fDefaultProvider", 0),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppSymmKeys",
                [],
                NDRFullPointerField(NDRByteField("ppSymmKeys", 0), deferred=True),
                size_is=lambda pkt: pkt.ulSymmKeysSize,
            ),
            deferred=True,
        ),
        NDRIntField("ulSymmKeysSize", None, size_of="ppSymmKeys"),
        NDRFullPointerField(NDRIntField("pulSymmKeysSizeProp", 0), deferred=True),
        NDRByteField("bEncrypted", 0),
        NDRByteField("bAuthenticated", 0),
        NDRShortField("uSenderIDLen", None, size_of="ppSenderID"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "ppSignature",
                [],
                NDRFullPointerField(NDRByteField("ppSignature", 0), deferred=True),
                size_is=lambda pkt: pkt.ulSignatureSize,
            ),
            deferred=True,
        ),
        NDRIntField("ulSignatureSize", None, size_of="ppSignature"),
        NDRFullPointerField(NDRIntField("pulSignatureSizeProp", 0), deferred=True),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppSrcQMID", GUID(), GUID), deferred=True
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRPacketField("pUow", XACTUOW(), XACTUOW), deferred=True),
        NDRFullPointerField(
            NDRConfVarFieldListField(
                "ppMsgExtension",
                [],
                NDRFullPointerField(
                    NDRVarStrLenField(
                        "ppMsgExtension",
                        "",
                        length_is=lambda pkt: pkt.ulMsgExtensionBufferInBytes,
                    ),
                    deferred=True,
                ),
                size_is=lambda pkt: pkt.ulMsgExtensionBufferInBytes,
                length_is=lambda pkt: pkt.ulMsgExtensionBufferInBytes,
            ),
            deferred=True,
        ),
        NDRIntField("ulMsgExtensionBufferInBytes", None, size_of="ppMsgExtension"),
        NDRFullPointerField(NDRIntField("pMsgExtensionSize", 0), deferred=True),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppConnectorType", GUID(), GUID), deferred=True
            ),
            deferred=True,
        ),
        NDRFullPointerField(NDRIntField("pulBodyType", 0), deferred=True),
        NDRFullPointerField(NDRIntField("pulVersion", 0), deferred=True),
    ]


class R_QMCreateRemoteCursor_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ptb1", CACTransferBufferV1(), CACTransferBufferV1),
        NDRIntField("hQueue", 0),
    ]


class R_QMCreateRemoteCursor_Response(NDRPacket):
    fields_desc = [NDRIntField("phCursor", 0), NDRIntField("status", 0)]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_LPWSTR = 31
    VT_BLOB = 65
    VT_CLSID = 72
    VT_VECTOR = 4096


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class ULARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("QuadPart", 0)]


class BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cbSize", None, size_of="pBlobData"),
        NDRFullPointerField(
            NDRConfStrLenField("pBlobData", "", size_is=lambda pkt: pkt.cbSize),
            deferred=True,
        ),
    ]


class CAUB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenField("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAUI(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("pElems", "", size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CAL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRSignedIntField("pElems", 0),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class CAUL(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems", [], NDRIntField("pElems", 0), size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CAUH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pElems", [], ULARGE_INTEGER, size_is=lambda pkt: pkt.cElems
            ),
            deferred=True,
        ),
    ]


class CACLSID(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfPacketListField("pElems", [], GUID, size_is=lambda pkt: pkt.cElems),
            deferred=True,
        ),
    ]


class CALPWSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cElems", None, size_of="pElems"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "pElems",
                [],
                NDRFullPointerField(
                    NDRConfVarStrNullFieldUtf16("pElems", ""), deferred=True
                ),
                size_is=lambda pkt: pkt.cElems,
            ),
            deferred=True,
        ),
    ]


class tag_inner_PROPVARIANT(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRShortField("vt", 0),
        NDRByteField("wReserved1", 0),
        NDRByteField("wReserved2", 0),
        NDRIntField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class R_QMCreateObjectInternal_Request(NDRPacket):
    fields_desc = [
        NDRIntField("dwObjectType", 0),
        NDRConfVarStrNullFieldUtf16("lpwcsPathName", ""),
        NDRIntField("SDSize", None, size_of="pSecurityDescriptor"),
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.SDSize),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class R_QMCreateObjectInternal_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class OBJECT_FORMAT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ObjType", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField("value", QUEUE_FORMAT(), QUEUE_FORMAT),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ObjType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                )
            ],
            StrFixedLenField("value", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class R_QMSetObjectSecurityInternal_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
        NDRIntField("SecurityInformation", 0),
        NDRIntField("SDSize", None, size_of="pSecurityDescriptor"),
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.SDSize),
    ]


class R_QMSetObjectSecurityInternal_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_QMGetObjectSecurityInternal_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
        NDRIntField("RequestedInformation", 0),
        NDRIntField("nLength", 0),
    ]


class R_QMGetObjectSecurityInternal_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pSecurityDescriptor", "", size_is=lambda pkt: pkt.nLength),
        NDRIntField("lpnLengthNeeded", 0),
        NDRIntField("status", 0),
    ]


class R_QMDeleteObject_Request(NDRPacket):
    fields_desc = [NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT)]


class R_QMDeleteObject_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_QMGetObjectProperties_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class R_QMGetObjectProperties_Response(NDRPacket):
    fields_desc = [
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
        NDRIntField("status", 0),
    ]


class R_QMSetObjectProperties_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
        NDRIntField("cp", None, size_of="apVar"),
        NDRConfFieldListField(
            "aProp", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cp
        ),
        NDRConfPacketListField(
            "apVar", [], tag_inner_PROPVARIANT, size_is=lambda pkt: pkt.cp
        ),
    ]


class R_QMSetObjectProperties_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_QMObjectPathToObjectFormat_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("lpwcsPathName", ""),
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
    ]


class R_QMObjectPathToObjectFormat_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pObjectFormat", OBJECT_FORMAT(), OBJECT_FORMAT),
        NDRIntField("status", 0),
    ]


class R_QMGetTmWhereabouts_Request(NDRPacket):
    fields_desc = [NDRIntField("cbBufSize", 0)]


class R_QMGetTmWhereabouts_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pbWhereabouts", "", size_is=lambda pkt: pkt.cbBufSize),
        NDRIntField("pcbWhereabouts", 0),
        NDRIntField("status", 0),
    ]


class R_QMEnlistTransaction_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pUow", XACTUOW(), XACTUOW),
        NDRIntField("cbCookie", None, size_of="pbCookie"),
        NDRConfStrLenField("pbCookie", "", size_is=lambda pkt: pkt.cbCookie),
    ]


class R_QMEnlistTransaction_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_QMEnlistInternalTransaction_Request(NDRPacket):
    fields_desc = [NDRPacketField("pUow", XACTUOW(), XACTUOW)]


class R_QMEnlistInternalTransaction_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phIntXact", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class R_QMCommitTransaction_Request(NDRPacket):
    fields_desc = [NDRPacketField("phIntXact", NDRContextHandle(), NDRContextHandle)]


class R_QMCommitTransaction_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phIntXact", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class R_QMAbortTransaction_Request(NDRPacket):
    fields_desc = [NDRPacketField("phIntXact", NDRContextHandle(), NDRContextHandle)]


class R_QMAbortTransaction_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phIntXact", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class rpc_QMOpenQueueInternal_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
        NDRIntField("dwDesiredAccess", 0),
        NDRIntField("dwShareMode", 0),
        NDRIntField("hRemoteQueue", 0),
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpRemoteQueueName", ""))
        ),
        NDRIntField("dwpQueue", 0),
        NDRPacketField("pLicGuid", GUID(), GUID),
        NDRConfVarStrNullFieldUtf16("lpClientName", ""),
        NDRIntField("dwRemoteProtocol", 0),
        NDRIntField("dwpRemoteContext", 0),
    ]


class rpc_QMOpenQueueInternal_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpRemoteQueueName", ""))
        ),
        NDRIntField("pdwQMContext", 0),
        NDRPacketField("phQueue", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class rpc_ACCloseHandle_Request(NDRPacket):
    fields_desc = [NDRPacketField("phQueue", NDRContextHandle(), NDRContextHandle)]


class rpc_ACCloseHandle_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phQueue", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class rpc_ACCloseCursor_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hQueue", NDRContextHandle(), NDRContextHandle),
        NDRIntField("hCursor", 0),
    ]


class rpc_ACCloseCursor_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class rpc_ACSetCursorProperties_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hProxy", NDRContextHandle(), NDRContextHandle),
        NDRIntField("hCursor", 0),
        NDRIntField("hRemoteCursor", 0),
    ]


class rpc_ACSetCursorProperties_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class rpc_ACHandleToFormatName_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hQueue", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwFormatNameRPCBufferLen", None, size_of="lpwcsFormatName"),
        NDRConfVarStrLenFieldUtf16(
            "lpwcsFormatName",
            "",
            size_is=lambda pkt: pkt.dwFormatNameRPCBufferLen,
            length_is=lambda pkt: pkt.dwFormatNameRPCBufferLen,
        ),
        NDRIntField("pdwLength", 0),
    ]


class rpc_ACHandleToFormatName_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenFieldUtf16(
            "lpwcsFormatName",
            "",
            size_is=lambda pkt: pkt.dwFormatNameRPCBufferLen,
            length_is=lambda pkt: pkt.dwFormatNameRPCBufferLen,
        ),
        NDRIntField("pdwLength", 0),
        NDRIntField("status", 0),
    ]


class rpc_ACPurgeQueue_Request(NDRPacket):
    fields_desc = [NDRPacketField("hQueue", NDRContextHandle(), NDRContextHandle)]


class rpc_ACPurgeQueue_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_QMQueryQMRegistryInternal_Request(NDRPacket):
    fields_desc = [NDRIntField("dwQueryType", 0)]


class R_QMQueryQMRegistryInternal_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("lplpMQISServer", "")),
        NDRIntField("status", 0),
    ]


class R_QMGetRTQMServerPort_Request(NDRPacket):
    fields_desc = [NDRIntField("fIP", 0)]


class R_QMGetRTQMServerPort_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


QMCOMM_OPNUMS = {  # 0: Opnum0NotUsedOnWire,
    1: DceRpcOp(R_QMGetRemoteQueueName_Request, R_QMGetRemoteQueueName_Response),
    2: DceRpcOp(R_QMOpenRemoteQueue_Request, R_QMOpenRemoteQueue_Response),
    3: DceRpcOp(
        R_QMCloseRemoteQueueContext_Request, R_QMCloseRemoteQueueContext_Response
    ),
    4: DceRpcOp(R_QMCreateRemoteCursor_Request, R_QMCreateRemoteCursor_Response),
    # 5: Opnum5NotUsedOnWire,
    6: DceRpcOp(R_QMCreateObjectInternal_Request, R_QMCreateObjectInternal_Response),
    7: DceRpcOp(
        R_QMSetObjectSecurityInternal_Request, R_QMSetObjectSecurityInternal_Response
    ),
    8: DceRpcOp(
        R_QMGetObjectSecurityInternal_Request, R_QMGetObjectSecurityInternal_Response
    ),
    9: DceRpcOp(R_QMDeleteObject_Request, R_QMDeleteObject_Response),
    10: DceRpcOp(R_QMGetObjectProperties_Request, R_QMGetObjectProperties_Response),
    11: DceRpcOp(R_QMSetObjectProperties_Request, R_QMSetObjectProperties_Response),
    12: DceRpcOp(
        R_QMObjectPathToObjectFormat_Request, R_QMObjectPathToObjectFormat_Response
    ),
    # 13: Opnum13NotUsedOnWire,
    14: DceRpcOp(R_QMGetTmWhereabouts_Request, R_QMGetTmWhereabouts_Response),
    15: DceRpcOp(R_QMEnlistTransaction_Request, R_QMEnlistTransaction_Response),
    16: DceRpcOp(
        R_QMEnlistInternalTransaction_Request, R_QMEnlistInternalTransaction_Response
    ),
    17: DceRpcOp(R_QMCommitTransaction_Request, R_QMCommitTransaction_Response),
    18: DceRpcOp(R_QMAbortTransaction_Request, R_QMAbortTransaction_Response),
    19: DceRpcOp(rpc_QMOpenQueueInternal_Request, rpc_QMOpenQueueInternal_Response),
    20: DceRpcOp(rpc_ACCloseHandle_Request, rpc_ACCloseHandle_Response),
    # 21: Opnum21NotUsedOnWire,
    22: DceRpcOp(rpc_ACCloseCursor_Request, rpc_ACCloseCursor_Response),
    23: DceRpcOp(rpc_ACSetCursorProperties_Request, rpc_ACSetCursorProperties_Response),
    # 24: Opnum24NotUsedOnWire,
    # 25: Opnum25NotUsedOnWire,
    26: DceRpcOp(rpc_ACHandleToFormatName_Request, rpc_ACHandleToFormatName_Response),
    27: DceRpcOp(rpc_ACPurgeQueue_Request, rpc_ACPurgeQueue_Response),
    28: DceRpcOp(
        R_QMQueryQMRegistryInternal_Request, R_QMQueryQMRegistryInternal_Response
    ),
    # 29: Opnum29NotUsedOnWire,
    # 30: Opnum30NotUsedOnWire,
    31: DceRpcOp(R_QMGetRTQMServerPort_Request, R_QMGetRTQMServerPort_Response),
    # 32: Opnum32NotUsedOnWire,
    # 33: Opnum33NotUsedOnWire,
    # 34: Opnum34NotUsedOnWire
}
register_dcerpc_interface(
    name="qmcomm",
    uuid=uuid.UUID("fdb3a030-065f-11d1-bb9b-00a024ea5525"),
    version="1.0",
    opnums=QMCOMM_OPNUMS,
)


class CACTransferBufferV2(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("old", CACTransferBufferV1(), CACTransferBufferV1),
        NDRFullPointerField(NDRByteField("pbFirstInXact", 0), deferred=True),
        NDRFullPointerField(NDRByteField("pbLastInXact", 0), deferred=True),
        NDRFullPointerField(
            NDRFullPointerField(
                NDRPacketField("ppXactID", OBJECTID(), OBJECTID), deferred=True
            ),
            deferred=True,
        ),
    ]


class QMSendMessageInternalEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pQueueFormat", QUEUE_FORMAT(), QUEUE_FORMAT),
        NDRPacketField("ptb", CACTransferBufferV2(), CACTransferBufferV2),
        NDRFullPointerField(NDRPacketField("pMessageID", OBJECTID(), OBJECTID)),
    ]


class QMSendMessageInternalEx_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pMessageID", OBJECTID(), OBJECTID)),
        NDRIntField("status", 0),
    ]


class rpc_ACSendMessageEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hQueue", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("ptb", CACTransferBufferV2(), CACTransferBufferV2),
        NDRFullPointerField(NDRPacketField("pMessageID", OBJECTID(), OBJECTID)),
    ]


class rpc_ACSendMessageEx_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("pMessageID", OBJECTID(), OBJECTID)),
        NDRIntField("status", 0),
    ]


class rpc_ACReceiveMessageEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hQMContext", 0),
        NDRPacketField("ptb", CACTransferBufferV2(), CACTransferBufferV2),
    ]


class rpc_ACReceiveMessageEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ptb", CACTransferBufferV2(), CACTransferBufferV2),
        NDRIntField("status", 0),
    ]


class rpc_ACCreateCursorEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("hQueue", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("pcc", CACCreateRemoteCursor(), CACCreateRemoteCursor),
    ]


class rpc_ACCreateCursorEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pcc", CACCreateRemoteCursor(), CACCreateRemoteCursor),
        NDRIntField("status", 0),
    ]


QMCOMM2_OPNUMS = {
    0: DceRpcOp(QMSendMessageInternalEx_Request, QMSendMessageInternalEx_Response),
    1: DceRpcOp(rpc_ACSendMessageEx_Request, rpc_ACSendMessageEx_Response),
    2: DceRpcOp(rpc_ACReceiveMessageEx_Request, rpc_ACReceiveMessageEx_Response),
    3: DceRpcOp(rpc_ACCreateCursorEx_Request, rpc_ACCreateCursorEx_Response),
}
register_dcerpc_interface(
    name="qmcomm2",
    uuid=uuid.UUID("76d12b80-3467-11d3-91ff-0090272f9ea3"),
    version="1.0",
    opnums=QMCOMM2_OPNUMS,
)



### ms-tsts_SessEnvRpc.idl
"""
RPC definitions for the following interfaces:
- SessEnvPublicRpc (v1.0): 1257B580-CE2F-4109-82D6-A9459D0BF6BC
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRInt3264EnumField,
    NDRIntField,
    register_dcerpc_interface,
)


class SHADOW_CONTROL_REQUEST(IntEnum):
    SHADOW_CONTROL_REQUEST_VIEW = 0
    SHADOW_CONTROL_REQUEST_TAKECONTROL = 1
    SHADOW_CONTROL_REQUEST_Count = 2


class SHADOW_PERMISSION_REQUEST(IntEnum):
    SHADOW_PERMISSION_REQUEST_SILENT = 0
    SHADOW_PERMISSION_REQUEST_REQUESTPERMISSION = 1
    SHADOW_PERMISSION_REQUEST_Count = 2


class SHADOW_REQUEST_RESPONSE(IntEnum):
    SHADOW_REQUEST_RESPONSE_ALLOW = 0
    SHADOW_REQUEST_RESPONSE_DECLINE = 1
    SHADOW_REQUEST_RESPONSE_POLICY_PERMISSION_REQUIRED = 2
    SHADOW_REQUEST_RESPONSE_POLICY_DISABLED = 3
    SHADOW_REQUEST_RESPONSE_POLICY_VIEW_ONLY = 4
    SHADOW_REQUEST_RESPONSE_POLICY_VIEW_ONLY_PERMISSION_REQUIRED = 5
    SHADOW_REQUEST_RESPONSE_SESSION_ALREADY_CONTROLLED = 6


class RpcShadow2_Request(NDRPacket):
    fields_desc = [
        NDRIntField("TargetSessionId", 0),
        NDRInt3264EnumField("eRequestControl", 0, SHADOW_CONTROL_REQUEST),
        NDRInt3264EnumField("eRequestPermission", 0, SHADOW_PERMISSION_REQUEST),
        NDRIntField("cchInvitation", 0),
    ]


class RpcShadow2_Response(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("pePermission", 0, SHADOW_REQUEST_RESPONSE),
        NDRConfVarStrNullFieldUtf16("pszInvitation", ""),
        NDRIntField("status", 0),
    ]


SESSENVPUBLICRPC_OPNUMS = {0: DceRpcOp(RpcShadow2_Request, RpcShadow2_Response)}
register_dcerpc_interface(
    name="SessEnvPublicRpc",
    uuid=uuid.UUID("1257B580-CE2F-4109-82D6-A9459D0BF6BC"),
    version="1.0",
    opnums=SESSENVPUBLICRPC_OPNUMS,
)



### ms-tsch_SchRpc.idl
"""
RPC definitions for the following interfaces:
- ITaskSchedulerService (v1.0): 86D35949-83C9-4044-B424-DB363231FD0C
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class SchRpcHighestVersion_Request(NDRPacket):
    fields_desc = []


class SchRpcHighestVersion_Response(NDRPacket):
    fields_desc = [NDRIntField("pVersion", 0), NDRIntField("status", 0)]


class TASK_USER_CRED(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("userId", ""), deferred=True),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("password", ""), deferred=True),
        NDRIntField("flags", 0),
    ]


class PTASK_XML_ERROR_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("line", 0),
        NDRIntField("column", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("node", ""), deferred=True),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("value", ""), deferred=True),
    ]


class SchRpcRegisterTask_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("path", "")),
        NDRConfVarStrNullFieldUtf16("xml", ""),
        NDRIntField("flags", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("sddl", "")),
        NDRIntField("logonType", 0),
        NDRIntField("cCreds", None, size_of="pCreds"),
        NDRConfPacketListField(
            "pCreds", [], TASK_USER_CRED, size_is=lambda pkt: pkt.cCreds
        ),
    ]


class SchRpcRegisterTask_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pActualPath", "")),
        NDRFullPointerField(
            NDRPacketField("pErrorInfo", PTASK_XML_ERROR_INFO(), PTASK_XML_ERROR_INFO)
        ),
        NDRIntField("status", 0),
    ]


class SchRpcRetrieveTask_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRConfVarStrNullFieldUtf16("lpcwszLanguagesBuffer", ""),
        NDRIntField("pulNumLanguages", 0),
    ]


class SchRpcRetrieveTask_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pXml", "")),
        NDRIntField("status", 0),
    ]


class SchRpcCreateFolder_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("sddl", "")),
        NDRIntField("flags", 0),
    ]


class SchRpcCreateFolder_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SchRpcSetSecurity_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRConfVarStrNullFieldUtf16("sddl", ""),
        NDRIntField("flags", 0),
    ]


class SchRpcSetSecurity_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SchRpcGetSecurity_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRIntField("securityInformation", 0),
    ]


class SchRpcGetSecurity_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("sddl", "")),
        NDRIntField("status", 0),
    ]


class SchRpcEnumFolders_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRIntField("flags", 0),
        NDRIntField("pStartIndex", 0),
        NDRIntField("cRequested", 0),
    ]


class SchRpcEnumFolders_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pStartIndex", 0),
        NDRIntField("pcNames", None, size_of="pNames"),
        NDRConfVarStrLenFieldUtf16("pNames", "", size_is=lambda pkt: pkt.pcNames),
        NDRIntField("status", 0),
    ]


class SchRpcEnumTasks_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRIntField("flags", 0),
        NDRIntField("startIndex", 0),
        NDRIntField("cRequested", 0),
    ]


class SchRpcEnumTasks_Response(NDRPacket):
    fields_desc = [
        NDRIntField("startIndex", 0),
        NDRIntField("pcNames", None, size_of="pNames"),
        NDRConfVarStrLenFieldUtf16("pNames", "", size_is=lambda pkt: pkt.pcNames),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class SchRpcEnumInstances_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("path", "")),
        NDRIntField("flags", 0),
    ]


class SchRpcEnumInstances_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcGuids", None, size_of="pGuids"),
        NDRConfPacketListField(
            "pGuids", [], GUID, size_is=lambda pkt: pkt.pcGuids, ptr_pack=True
        ),
        NDRIntField("status", 0),
    ]


class SchRpcGetInstanceInfo_Request(NDRPacket):
    fields_desc = [NDRPacketField("guid", GUID(), GUID)]


class SchRpcGetInstanceInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pPath", "")),
        NDRIntField("pState", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pCurrentAction", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pInfo", "")),
        NDRIntField("pcGroupInstances", None, size_of="pGroupInstances"),
        NDRConfPacketListField(
            "pGroupInstances",
            [],
            GUID,
            size_is=lambda pkt: pkt.pcGroupInstances,
            ptr_pack=True,
        ),
        NDRIntField("pEnginePID", 0),
        NDRIntField("status", 0),
    ]


class SchRpcStopInstance_Request(NDRPacket):
    fields_desc = [NDRPacketField("guid", GUID(), GUID), NDRIntField("flags", 0)]


class SchRpcStopInstance_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SchRpcStop_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("path", "")),
        NDRIntField("flags", 0),
    ]


class SchRpcStop_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SchRpcRun_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRIntField("cArgs", None, size_of="pArgs"),
        NDRConfVarStrLenFieldUtf16("pArgs", "", size_is=lambda pkt: pkt.cArgs),
        NDRIntField("flags", 0),
        NDRIntField("sessionId", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("user", "")),
    ]


class SchRpcRun_Response(NDRPacket):
    fields_desc = [NDRPacketField("pGuid", GUID(), GUID), NDRIntField("status", 0)]


class SchRpcDelete_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("path", ""), NDRIntField("flags", 0)]


class SchRpcDelete_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SchRpcRename_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRConfVarStrNullFieldUtf16("newName", ""),
        NDRIntField("flags", 0),
    ]


class SchRpcRename_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PSYSTEMTIME(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRShortField("wYear", 0),
        NDRShortField("wMonth", 0),
        NDRShortField("wDayOfWeek", 0),
        NDRShortField("wDay", 0),
        NDRShortField("wHour", 0),
        NDRShortField("wMinute", 0),
        NDRShortField("wSecond", 0),
        NDRShortField("wMilliseconds", 0),
    ]


class SchRpcScheduledRuntimes_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("path", ""),
        NDRFullPointerField(NDRPacketField("start", PSYSTEMTIME(), PSYSTEMTIME)),
        NDRFullPointerField(NDRPacketField("end", PSYSTEMTIME(), PSYSTEMTIME)),
        NDRIntField("flags", 0),
        NDRIntField("cRequested", 0),
    ]


class SchRpcScheduledRuntimes_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcRuntimes", None, size_of="pRuntimes"),
        NDRConfPacketListField(
            "pRuntimes",
            [],
            PSYSTEMTIME,
            size_is=lambda pkt: pkt.pcRuntimes,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class SchRpcGetLastRunInfo_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("path", "")]


class SchRpcGetLastRunInfo_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pLastRuntime", PSYSTEMTIME(), PSYSTEMTIME),
        NDRIntField("pLastReturnCode", 0),
        NDRIntField("status", 0),
    ]


class SchRpcGetTaskInfo_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("path", ""), NDRIntField("flags", 0)]


class SchRpcGetTaskInfo_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pEnabled", 0),
        NDRIntField("pState", 0),
        NDRIntField("status", 0),
    ]


class SchRpcGetNumberOfMissedRuns_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("path", "")]


class SchRpcGetNumberOfMissedRuns_Response(NDRPacket):
    fields_desc = [NDRIntField("pNumberOfMissedRuns", 0), NDRIntField("status", 0)]


class SchRpcEnableTask_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("path", ""), NDRIntField("enabled", 0)]


class SchRpcEnableTask_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


ITASKSCHEDULERSERVICE_OPNUMS = {
    0: DceRpcOp(SchRpcHighestVersion_Request, SchRpcHighestVersion_Response),
    1: DceRpcOp(SchRpcRegisterTask_Request, SchRpcRegisterTask_Response),
    2: DceRpcOp(SchRpcRetrieveTask_Request, SchRpcRetrieveTask_Response),
    3: DceRpcOp(SchRpcCreateFolder_Request, SchRpcCreateFolder_Response),
    4: DceRpcOp(SchRpcSetSecurity_Request, SchRpcSetSecurity_Response),
    5: DceRpcOp(SchRpcGetSecurity_Request, SchRpcGetSecurity_Response),
    6: DceRpcOp(SchRpcEnumFolders_Request, SchRpcEnumFolders_Response),
    7: DceRpcOp(SchRpcEnumTasks_Request, SchRpcEnumTasks_Response),
    8: DceRpcOp(SchRpcEnumInstances_Request, SchRpcEnumInstances_Response),
    9: DceRpcOp(SchRpcGetInstanceInfo_Request, SchRpcGetInstanceInfo_Response),
    10: DceRpcOp(SchRpcStopInstance_Request, SchRpcStopInstance_Response),
    11: DceRpcOp(SchRpcStop_Request, SchRpcStop_Response),
    12: DceRpcOp(SchRpcRun_Request, SchRpcRun_Response),
    13: DceRpcOp(SchRpcDelete_Request, SchRpcDelete_Response),
    14: DceRpcOp(SchRpcRename_Request, SchRpcRename_Response),
    15: DceRpcOp(SchRpcScheduledRuntimes_Request, SchRpcScheduledRuntimes_Response),
    16: DceRpcOp(SchRpcGetLastRunInfo_Request, SchRpcGetLastRunInfo_Response),
    17: DceRpcOp(SchRpcGetTaskInfo_Request, SchRpcGetTaskInfo_Response),
    18: DceRpcOp(
        SchRpcGetNumberOfMissedRuns_Request, SchRpcGetNumberOfMissedRuns_Response
    ),
    19: DceRpcOp(SchRpcEnableTask_Request, SchRpcEnableTask_Response),
}
register_dcerpc_interface(
    name="ITaskSchedulerService",
    uuid=uuid.UUID("86D35949-83C9-4044-B424-DB363231FD0C"),
    version="1.0",
    opnums=ITASKSCHEDULERSERVICE_OPNUMS,
)



### ms-dtyp.idl
"""
RPC definitions for the following interfaces:
- 
This file is auto-generated by midl-to-scapy, do not modify.
"""


import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
)


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class PFILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class LPFILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class UUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class PGUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class PLARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class EVENT_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRShortField("Id", 0),
        NDRByteField("Version", 0),
        NDRByteField("Channel", 0),
        NDRByteField("Level", 0),
        NDRByteField("Opcode", 0),
        NDRShortField("Task", 0),
        NDRLongField("Keyword", 0),
    ]


class PEVENT_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRShortField("Id", 0),
        NDRByteField("Version", 0),
        NDRByteField("Channel", 0),
        NDRByteField("Level", 0),
        NDRByteField("Opcode", 0),
        NDRShortField("Task", 0),
        NDRLongField("Keyword", 0),
    ]


class PCEVENT_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRShortField("Id", 0),
        NDRByteField("Version", 0),
        NDRByteField("Channel", 0),
        NDRByteField("Level", 0),
        NDRByteField("Opcode", 0),
        NDRShortField("Task", 0),
        NDRLongField("Keyword", 0),
    ]


class LUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("LowPart", 0), NDRSignedIntField("HighPart", 0)]


class PLUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("LowPart", 0), NDRSignedIntField("HighPart", 0)]


class MULTI_SZ(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(NDRShortField("Value", 0), deferred=True),
        NDRIntField("nChar", 0),
    ]


class RPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class PRPC_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class SERVER_INFO_100(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv100_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv100_name", ""), deferred=True
        ),
    ]


class PSERVER_INFO_100(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv100_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv100_name", ""), deferred=True
        ),
    ]


class LPSERVER_INFO_100(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv100_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv100_name", ""), deferred=True
        ),
    ]


class SERVER_INFO_101(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv101_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_name", ""), deferred=True
        ),
        NDRIntField("sv101_version_major", 0),
        NDRIntField("sv101_version_minor", 0),
        NDRIntField("sv101_version_type", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_comment", ""), deferred=True
        ),
    ]


class PSERVER_INFO_101(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv101_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_name", ""), deferred=True
        ),
        NDRIntField("sv101_version_major", 0),
        NDRIntField("sv101_version_minor", 0),
        NDRIntField("sv101_version_type", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_comment", ""), deferred=True
        ),
    ]


class LPSERVER_INFO_101(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv101_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_name", ""), deferred=True
        ),
        NDRIntField("sv101_version_major", 0),
        NDRIntField("sv101_version_minor", 0),
        NDRIntField("sv101_version_type", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv101_comment", ""), deferred=True
        ),
    ]


class SYSTEMTIME(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRShortField("wYear", 0),
        NDRShortField("wMonth", 0),
        NDRShortField("wDayOfWeek", 0),
        NDRShortField("wDay", 0),
        NDRShortField("wHour", 0),
        NDRShortField("wMinute", 0),
        NDRShortField("wSecond", 0),
        NDRShortField("wMilliseconds", 0),
    ]


class PSYSTEMTIME(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRShortField("wYear", 0),
        NDRShortField("wMonth", 0),
        NDRShortField("wDayOfWeek", 0),
        NDRShortField("wDay", 0),
        NDRShortField("wHour", 0),
        NDRShortField("wMinute", 0),
        NDRShortField("wSecond", 0),
        NDRShortField("wMilliseconds", 0),
    ]


class UINT128(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("lower", 0), NDRLongField("upper", 0)]


class PUINT128(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("lower", 0), NDRLongField("upper", 0)]


class ULARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("QuadPart", 0)]


class PULARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("QuadPart", 0)]


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class OBJECT_TYPE_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Level", 0),
        NDRIntField("Remaining", 0),
        NDRFullPointerField(NDRPacketField("ObjectType", GUID(), GUID), deferred=True),
    ]


class POBJECT_TYPE_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Level", 0),
        NDRIntField("Remaining", 0),
        NDRFullPointerField(NDRPacketField("ObjectType", GUID(), GUID), deferred=True),
    ]


class ACE_HEADER(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRByteField("AceType", 0),
        NDRByteField("AceFlags", 0),
        NDRShortField("AceSize", 0),
    ]


class PACE_HEADER(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRByteField("AceType", 0),
        NDRByteField("AceFlags", 0),
        NDRShortField("AceSize", 0),
    ]


class SYSTEM_MANDATORY_LABEL_ACE(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Header", ACE_HEADER(), ACE_HEADER),
        NDRIntField("Mask", 0),
        NDRIntField("SidStart", 0),
    ]


class PSYSTEM_MANDATORY_LABEL_ACE(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("Header", ACE_HEADER(), ACE_HEADER),
        NDRIntField("Mask", 0),
        NDRIntField("SidStart", 0),
    ]


class TOKEN_MANDATORY_POLICY(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("Policy", 0)]


class PTOKEN_MANDATORY_POLICY(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("Policy", 0)]


class MANDATORY_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("AllowedAccess", 0),
        NDRByteField("WriteAllowed", 0),
        NDRByteField("ReadAllowed", 0),
        NDRByteField("ExecuteAllowed", 0),
        NDRPacketField(
            "MandatoryPolicy", TOKEN_MANDATORY_POLICY(), TOKEN_MANDATORY_POLICY
        ),
    ]


class PMANDATORY_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("AllowedAccess", 0),
        NDRByteField("WriteAllowed", 0),
        NDRByteField("ReadAllowed", 0),
        NDRByteField("ExecuteAllowed", 0),
        NDRPacketField(
            "MandatoryPolicy", TOKEN_MANDATORY_POLICY(), TOKEN_MANDATORY_POLICY
        ),
    ]


class RPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class PRPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class PSID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class ACL(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRByteField("AclRevision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("AclSize", 0),
        NDRShortField("AceCount", 0),
        NDRShortField("Sbz2", 0),
    ]


class PACL(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRByteField("AclRevision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("AclSize", 0),
        NDRShortField("AceCount", 0),
        NDRShortField("Sbz2", 0),
    ]


class SECURITY_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("Control", 0),
        NDRFullPointerField(NDRPacketField("Owner", PSID(), PSID), deferred=True),
        NDRFullPointerField(NDRPacketField("Group", PSID(), PSID), deferred=True),
        NDRFullPointerField(NDRPacketField("Sacl", PACL(), PACL), deferred=True),
        NDRFullPointerField(NDRPacketField("Dacl", PACL(), PACL), deferred=True),
    ]


class PSECURITY_DESCRIPTOR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("Sbz1", 0),
        NDRShortField("Control", 0),
        NDRFullPointerField(NDRPacketField("Owner", PSID(), PSID), deferred=True),
        NDRFullPointerField(NDRPacketField("Group", PSID(), PSID), deferred=True),
        NDRFullPointerField(NDRPacketField("Sacl", PACL(), PACL), deferred=True),
        NDRFullPointerField(NDRPacketField("Dacl", PACL(), PACL), deferred=True),
    ]



### ms-csra.idl
"""
RPC definitions for the following interfaces:
- ICertAdminD (v0.0): d99e6e71-fc88-11d0-b498-00a0c90312f3
- ICertAdminD2 (v0.0): 7fe0d935-dda6-443f-85d0-1cfb58fe41dd
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRRecursiveField,
    NDRRefEmbPointerField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    register_com_interface,
)


class CERTTRANSBLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cb", None, size_of="pb"),
        NDRFullPointerField(
            NDRConfStrLenField("pb", "", size_is=lambda pkt: pkt.cb), deferred=True
        ),
    ]


class SetExtension_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszExtensionName", "")),
        NDRIntField("dwType", 0),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pctbValue", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class SetExtension_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SetAttributes_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAttributes", "")),
    ]


class SetAttributes_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ResubmitRequest_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
    ]


class ResubmitRequest_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwDisposition", 0), NDRIntField("status", 0)]


class DenyRequest_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
    ]


class DenyRequest_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsValidCertificate_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pSerialNumber", "")),
    ]


class IsValidCertificate_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pRevocationReason", 0),
        NDRSignedIntField("pDisposition", 0),
        NDRIntField("status", 0),
    ]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class PublishCRL_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRPacketField("FileTime", FILETIME(), FILETIME),
    ]


class PublishCRL_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetCRL_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetCRL_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pctbCRL", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class RevokeCertificate_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszSerialNumber", "")),
        NDRIntField("Reason", 0),
        NDRPacketField("FileTime", FILETIME(), FILETIME),
    ]


class RevokeCertificate_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumViewColumn_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("iColumn", 0),
        NDRIntField("cColumn", 0),
    ]


class EnumViewColumn_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcColumn", 0),
        NDRPacketField("pctbColumnInfo", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class GetViewDefaultColumnSet_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("iColumnSetDefault", 0),
    ]


class GetViewDefaultColumnSet_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcColumn", 0),
        NDRPacketField("pctbColumnInfo", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class EnumAttributesOrExtensions_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("RowId", 0),
        NDRIntField("Flags", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszLast", "")),
        NDRIntField("celt", 0),
    ]


class EnumAttributesOrExtensions_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pceltFetched", 0),
        NDRPacketField("pctbOut", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class CERTVIEWRESTRICTION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ColumnIndex", 0),
        NDRSignedIntField("SeekOperator", 0),
        NDRSignedIntField("SortOrder", 0),
        NDRFullPointerField(
            NDRConfStrLenField("pbValue", "", size_is=lambda pkt: pkt.cbValue),
            deferred=True,
        ),
        NDRIntField("cbValue", None, size_of="pbValue"),
    ]


class OpenView_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("ccvr", None, size_of="acvr"),
        NDRConfPacketListField(
            "acvr", [], CERTVIEWRESTRICTION, size_is=lambda pkt: pkt.ccvr
        ),
        NDRIntField("ccolOut", None, size_of="acolOut"),
        NDRConfFieldListField(
            "acolOut", [], NDRIntField("", 0), size_is=lambda pkt: pkt.ccolOut
        ),
        NDRIntField("ielt", 0),
        NDRIntField("celt", 0),
    ]


class OpenView_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pceltFetched", 0),
        NDRPacketField("pctbResultRows", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class EnumView_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("ielt", 0),
        NDRIntField("celt", 0),
    ]


class EnumView_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pceltFetched", 0),
        NDRPacketField("pctbResultRows", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class CloseView_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class CloseView_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ServerControl_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwControlFlags", 0),
    ]


class ServerControl_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pctbOut", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class Ping_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class Ping_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetServerState_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetServerState_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwState", 0), NDRIntField("status", 0)]


class BackupPrepare_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("grbitJet", 0),
        NDRIntField("dwBackupFlags", 0),
        NDRShortField("pwszBackupAnnotation", 0),
        NDRIntField("dwClientIdentifier", 0),
    ]


class BackupPrepare_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BackupEnd_Request(NDRPacket):
    fields_desc = []


class BackupEnd_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BackupGetAttachmentInformation_Request(NDRPacket):
    fields_desc = []


class BackupGetAttachmentInformation_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "ppwszzDBFiles", "", size_is=lambda pkt: pkt.pcwcDBFiles
        ),
        NDRSignedIntField("pcwcDBFiles", None, size_of="ppwszzDBFiles"),
        NDRIntField("status", 0),
    ]


class BackupGetBackupLogs_Request(NDRPacket):
    fields_desc = []


class BackupGetBackupLogs_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "ppwszzLogFiles", "", size_is=lambda pkt: pkt.pcwcLogFiles
        ),
        NDRSignedIntField("pcwcLogFiles", None, size_of="ppwszzLogFiles"),
        NDRIntField("status", 0),
    ]


class BackupOpenFile_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszPath", ""))]


class BackupOpenFile_Response(NDRPacket):
    fields_desc = [NDRLongField("pliLength", 0), NDRIntField("status", 0)]


class BackupReadFile_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("cbBuffer", 0)]


class BackupReadFile_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pbBuffer", "", size_is=lambda pkt: pkt.cbBuffer),
        NDRSignedIntField("pcbRead", 0),
        NDRIntField("status", 0),
    ]


class BackupCloseFile_Request(NDRPacket):
    fields_desc = []


class BackupCloseFile_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BackupTruncateLogs_Request(NDRPacket):
    fields_desc = []


class BackupTruncateLogs_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ImportCertificate_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRPacketField("pctbCertificate", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRSignedIntField("dwFlags", 0),
    ]


class ImportCertificate_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pdwRequestId", 0), NDRIntField("status", 0)]


class BackupGetDynamicFiles_Request(NDRPacket):
    fields_desc = []


class BackupGetDynamicFiles_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16("ppwszzFiles", "", size_is=lambda pkt: pkt.pcwcFiles),
        NDRSignedIntField("pcwcFiles", None, size_of="ppwszzFiles"),
        NDRIntField("status", 0),
    ]


class RestoreGetDatabaseLocations_Request(NDRPacket):
    fields_desc = []


class RestoreGetDatabaseLocations_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "ppwszzDatabaseLocations", "", size_is=lambda pkt: pkt.pcwcPaths
        ),
        NDRSignedIntField("pcwcPaths", None, size_of="ppwszzDatabaseLocations"),
        NDRIntField("status", 0),
    ]


ICERTADMIND_OPNUMS = {
    0: DceRpcOp(SetExtension_Request, SetExtension_Response),
    1: DceRpcOp(SetAttributes_Request, SetAttributes_Response),
    2: DceRpcOp(ResubmitRequest_Request, ResubmitRequest_Response),
    3: DceRpcOp(DenyRequest_Request, DenyRequest_Response),
    4: DceRpcOp(IsValidCertificate_Request, IsValidCertificate_Response),
    5: DceRpcOp(PublishCRL_Request, PublishCRL_Response),
    6: DceRpcOp(GetCRL_Request, GetCRL_Response),
    7: DceRpcOp(RevokeCertificate_Request, RevokeCertificate_Response),
    8: DceRpcOp(EnumViewColumn_Request, EnumViewColumn_Response),
    9: DceRpcOp(GetViewDefaultColumnSet_Request, GetViewDefaultColumnSet_Response),
    10: DceRpcOp(
        EnumAttributesOrExtensions_Request, EnumAttributesOrExtensions_Response
    ),
    11: DceRpcOp(OpenView_Request, OpenView_Response),
    12: DceRpcOp(EnumView_Request, EnumView_Response),
    13: DceRpcOp(CloseView_Request, CloseView_Response),
    14: DceRpcOp(ServerControl_Request, ServerControl_Response),
    15: DceRpcOp(Ping_Request, Ping_Response),
    16: DceRpcOp(GetServerState_Request, GetServerState_Response),
    17: DceRpcOp(BackupPrepare_Request, BackupPrepare_Response),
    18: DceRpcOp(BackupEnd_Request, BackupEnd_Response),
    19: DceRpcOp(
        BackupGetAttachmentInformation_Request, BackupGetAttachmentInformation_Response
    ),
    20: DceRpcOp(BackupGetBackupLogs_Request, BackupGetBackupLogs_Response),
    21: DceRpcOp(BackupOpenFile_Request, BackupOpenFile_Response),
    22: DceRpcOp(BackupReadFile_Request, BackupReadFile_Response),
    23: DceRpcOp(BackupCloseFile_Request, BackupCloseFile_Response),
    24: DceRpcOp(BackupTruncateLogs_Request, BackupTruncateLogs_Response),
    25: DceRpcOp(ImportCertificate_Request, ImportCertificate_Response),
    26: DceRpcOp(BackupGetDynamicFiles_Request, BackupGetDynamicFiles_Response),
    27: DceRpcOp(
        RestoreGetDatabaseLocations_Request, RestoreGetDatabaseLocations_Response
    ),
}
register_com_interface(
    name="ICertAdminD",
    uuid=uuid.UUID("d99e6e71-fc88-11d0-b498-00a0c90312f3"),
    opnums=ICERTADMIND_OPNUMS,
)


class PublishCRLs_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRPacketField("FileTime", FILETIME(), FILETIME),
        NDRIntField("Flags", 0),
    ]


class PublishCRLs_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetCAProperty_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRSignedIntField("PropId", 0),
        NDRSignedIntField("PropIndex", 0),
        NDRSignedIntField("PropType", 0),
    ]


class GetCAProperty_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pctbPropertyValue", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class SetCAProperty_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRSignedIntField("PropId", 0),
        NDRSignedIntField("PropIndex", 0),
        NDRSignedIntField("PropType", 0),
        NDRPacketField("pctbPropertyValue", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class SetCAProperty_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetCAPropertyInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetCAPropertyInfo_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pcProperty", 0),
        NDRPacketField("pctbPropInfo", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class EnumViewColumnTable_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("iTable", 0),
        NDRIntField("iColumn", 0),
        NDRIntField("cColumn", 0),
    ]


class EnumViewColumnTable_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcColumn", 0),
        NDRPacketField("pctbColumnInfo", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class GetCASecurity_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetCASecurity_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pctbSD", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class SetCASecurity_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRPacketField("pctbSD", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class SetCASecurity_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Ping2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class Ping2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetArchivedKey_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
    ]


class GetArchivedKey_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pctbArchivedKey", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class GetAuditFilter_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetAuditFilter_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwFilter", 0), NDRIntField("status", 0)]


class SetAuditFilter_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwFilter", 0),
    ]


class SetAuditFilter_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetOfficerRights_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetOfficerRights_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pfEnabled", 0),
        NDRPacketField("pctbSD", CERTTRANSBLOB(), CERTTRANSBLOB),
        NDRIntField("status", 0),
    ]


class SetOfficerRights_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRSignedIntField("fEnable", 0),
        NDRPacketField("pctbSD", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class SetOfficerRights_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class CURRENCY(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("int64", 0)]


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class SF_TYPE(IntEnum):
    SF_ERROR = VARENUM.VT_ERROR
    SF_I1 = VARENUM.VT_I1
    SF_I2 = VARENUM.VT_I2
    SF_I4 = VARENUM.VT_I4
    SF_I8 = VARENUM.VT_I8
    SF_BSTR = VARENUM.VT_BSTR
    SF_UNKNOWN = VARENUM.VT_UNKNOWN
    SF_DISPATCH = VARENUM.VT_DISPATCH
    SF_VARIANT = VARENUM.VT_VARIANT
    SF_RECORD = VARENUM.VT_RECORD
    SF_HAVEIID = VARENUM.VT_UNKNOWN | 32768


class SAFEARR_BSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="aBstr"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "aBstr", [], FLAGGED_WORD_BLOB, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_UNKNOWN(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apUnknown"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apUnknown", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_DISPATCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apDispatch"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apDispatch", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class wireVARIANTStr(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("clSize", 0),
        NDRIntField("rpcReserved", 0),
        NDRShortField("vt", 0),
        NDRShortField("wReserved1", 0),
        NDRShortField("wReserved2", 0),
        NDRShortField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class GetConfigEntry_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszNodePath", "")),
        NDRConfVarStrNullFieldUtf16("pwszEntry", ""),
    ]


class GetConfigEntry_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVariant", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class SetConfigEntry_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszNodePath", "")),
        NDRConfVarStrNullFieldUtf16("pwszEntry", ""),
        NDRFullPointerField(
            NDRPacketField("pVariant", wireVARIANTStr(), wireVARIANTStr)
        ),
    ]


class SetConfigEntry_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ImportKey_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwRequestId", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszCertHash", "")),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pctbKey", CERTTRANSBLOB(), CERTTRANSBLOB),
    ]


class ImportKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetMyRoles_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", ""))
    ]


class GetMyRoles_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pdwRoles", 0), NDRIntField("status", 0)]


class DeleteRow_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszAuthority", "")),
        NDRIntField("dwFlags", 0),
        NDRPacketField("FileTime", FILETIME(), FILETIME),
        NDRIntField("dwTable", 0),
        NDRIntField("dwRowId", 0),
    ]


class DeleteRow_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pcDeleted", 0), NDRIntField("status", 0)]


ICERTADMIND2_OPNUMS = {
    0: DceRpcOp(PublishCRLs_Request, PublishCRLs_Response),
    1: DceRpcOp(GetCAProperty_Request, GetCAProperty_Response),
    2: DceRpcOp(SetCAProperty_Request, SetCAProperty_Response),
    3: DceRpcOp(GetCAPropertyInfo_Request, GetCAPropertyInfo_Response),
    4: DceRpcOp(EnumViewColumnTable_Request, EnumViewColumnTable_Response),
    5: DceRpcOp(GetCASecurity_Request, GetCASecurity_Response),
    6: DceRpcOp(SetCASecurity_Request, SetCASecurity_Response),
    7: DceRpcOp(Ping2_Request, Ping2_Response),
    8: DceRpcOp(GetArchivedKey_Request, GetArchivedKey_Response),
    9: DceRpcOp(GetAuditFilter_Request, GetAuditFilter_Response),
    10: DceRpcOp(SetAuditFilter_Request, SetAuditFilter_Response),
    11: DceRpcOp(GetOfficerRights_Request, GetOfficerRights_Response),
    12: DceRpcOp(SetOfficerRights_Request, SetOfficerRights_Response),
    13: DceRpcOp(GetConfigEntry_Request, GetConfigEntry_Response),
    14: DceRpcOp(SetConfigEntry_Request, SetConfigEntry_Response),
    15: DceRpcOp(ImportKey_Request, ImportKey_Response),
    16: DceRpcOp(GetMyRoles_Request, GetMyRoles_Response),
    17: DceRpcOp(DeleteRow_Request, DeleteRow_Response),
}
register_com_interface(
    name="ICertAdminD2",
    uuid=uuid.UUID("7fe0d935-dda6-443f-85d0-1cfb58fe41dd"),
    opnums=ICERTADMIND2_OPNUMS,
)



### ms-rsp_windowsshutdown.idl
"""
RPC definitions for the following interfaces:
- WindowsShutdown (v1.0): d95afe70-a6d5-4259-822e-2c84da1ddb0d
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class PREG_UNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class WsdrInitiateShutdown_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("lpMessage", PREG_UNICODE_STRING(), PREG_UNICODE_STRING)
        ),
        NDRIntField("dwGracePeriod", 0),
        NDRIntField("dwShudownFlags", 0),
        NDRIntField("dwReason", 0),
        NDRFullPointerField(
            NDRPacketField("lpClientHint", PREG_UNICODE_STRING(), PREG_UNICODE_STRING)
        ),
    ]


class WsdrInitiateShutdown_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class WsdrAbortShutdown_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("lpClientHint", PREG_UNICODE_STRING(), PREG_UNICODE_STRING)
        )
    ]


class WsdrAbortShutdown_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


WINDOWSSHUTDOWN_OPNUMS = {
    0: DceRpcOp(WsdrInitiateShutdown_Request, WsdrInitiateShutdown_Response),
    1: DceRpcOp(WsdrAbortShutdown_Request, WsdrAbortShutdown_Response),
}
register_dcerpc_interface(
    name="WindowsShutdown",
    uuid=uuid.UUID("d95afe70-a6d5-4259-822e-2c84da1ddb0d"),
    version="1.0",
    opnums=WINDOWSSHUTDOWN_OPNUMS,
)
